% Michael Vodhanel

% Purpose: Used ICA to find independent components of DNA samples known
%				to contan cancer and compares the components with an unused
%				set of DNA samples known to have cancer and a set that is known
%				to be cancer free to see if the independent components are
%				indicators of bladder cancer.

% Input:
%	BladderData: The Bladder DNA data set 

% Output:
%	CharTumor: The characteristic images (modes) of the DNA data set with cancer to the screen
%	U: The independent components of the modes found by ICA to the screen

clear all;

% Load data from file
fprintf('Loading cancer data\n\n')
%BladderData = load('imputed.txt');
BladderData = load('Bladder_Data.txt');
%BladderData = load('liver_cancer_XICHEN_12405.txt');
DNA_Size = size(BladderData, 1);

% seperate data into normal and tumor sets
NormalIndex=[103:124];
TumorIndexAll=[1:6,8:102,125,126];
NumOfNormal = length(NormalIndex)
NumOfTumor = length(TumorIndexAll)

Normal = BladderData(:, NormalIndex);
Tumor = BladderData(:, TumorIndexAll);
clear BladderData;

% Make up a draw height and width for viewing results
Height = 6688 / 32;
Width = 6688 / 209;

% Get four characteristic images for the tumor data 
% using zero noise, the lump scheme, and 50 iterations
fprintf('Finding characteristics images for tumors \n\n')
CharTumor = zeros(DNA_Size, 4);
%Ind(1,:) = [1:25]; %[1:floor(NumOfTumor/4)];
%Ind(2,:) = [26:50]; %[floor(NumOfTumor/4)+1:floor(NumOfTumor/2)];
%Ind(3,:) = [51:75]; %[floor(NumOfTumor/2)+1:floor(3*NumOfTumor/4)];
%Ind(4,:) = [76:100]; %[floor(3*NumOfTumor/4)+1:NumOfTumor];
Ind = reshape([1:80]', 5, 16);
Ns = size(Ind, 1);
for i = 1:Ns
	CharTumor(:, i) = POD(Tumor(:, Ind(i,:)), zeros(size(Tumor(:, Ind(i,:)))), 'L', 50);
end
%CharTumor(:, 2) = POD(Tumor(:, Ind2), zeros(size(Tumor(:, Ind(1,:)))), 'p', 50);
%CharTumor(:, 3) = POD(Tumor(:, Ind3), zeros(size(Tumor(:, Ind(1,:)))), 'p', 50);
%CharTumor(:, 4) = POD(Tumor(:, Ind4), zeros(size(Tumor(:, Ind4))), 'p', 50);

% Draw characteristic images which are also the signals into ICA
figure(1);clf
for i=1:Ns
    subplot(4, 2, i);
    Title = ['Signal ' num2str(i)];
    Draw_Sub_Fig(CharTumor(:,i), Height, Width, Title, 'f');
end

% Use ICA to find the independent components
% Transpose the data since ICA uses rows instead of columns for input
fprintf('Performing ICA \n\n')
% TRY FASTICA here   %nabbasi 032007
U = ICA(CharTumor');

% Draw independent components
figure(2);clf
for i=1:Ns
    subplot(4, 2, i);
    Title = ['Inpendent Component ' num2str(i)];
    Draw_Sub_Fig(U(i,:), Height, Width, Title, 'f');
 end
 
 % Make tumor vs normal comparison matrix to independent components
 CompareAvg = zeros(Ns, 2);
 for i = 1:Ns
    for j=1:NumOfNormal
       CompareAvg(i, 1) = CompareAvg(i, 1) + dot(U(i,:), Tumor(:,80+j)') / (norm(U(i,:)) * norm(Tumor(:,80+j)));
    end
    %CompareAvg(i, 1) = mean([dot(U(i,:), Tumor(:,101)'), dot(U(i,:), Tumor(:,102)'), dot(U(i,:), Tumor(:,103)')]);
    CompareAvg(i, 1) = CompareAvg(i, 1) / NumOfNormal;
    for j=1:NumOfNormal
       CompareAvg(i, 2) = CompareAvg(i, 2) + dot(U(i,:), Normal(:,j)') / (norm(U(i,:)) * norm(Normal(:,j)));
    end
    CompareAvg(i, 2) = CompareAvg(i, 2) / NumOfNormal;
 end
 CompareAvg = CompareAvg
 
 CompareCanc = zeros(Ns, NumOfNormal);
 CompareNorm = zeros(Ns, NumOfNormal);
 for i = 1:Ns
     for j = 1:NumOfNormal
         CompareCanc(i,j) = dot(U(i,:), Tumor(:,80+j)') / (norm(U(i,:)) * norm(Tumor(:,80+j)));
         CompareNorm(i,j) = dot(U(i,:), Normal(:,j)') / (norm(U(i,:)) * norm(Normal(:,j)));
     end
 end
 Tum_Mean = mean(CompareCanc')
 Tum_StdDev = std(CompareCanc')
 Tum_Skew = sum((CompareCanc' - repmat(Tum_Mean, 22, 1)).^3) ./ Tum_StdDev.^3
 Norm_Mean = mean(CompareNorm')
 Norm_StdDev = std(CompareNorm')
 Norm_Skew = sum((CompareNorm' - repmat(Norm_Mean, 22, 1)).^3) ./ Norm_StdDev.^3
 
 
 